@@ -236,9 +236,11 @@ panel_dock_get_child_position_cb (PanelDock     *self,
    * adjust the margin/padding/etc.
    */
   gtk_widget_get_preferred_size (child, &min, &nat);
-  gtk_widget_get_allocation (GTK_WIDGET (self), allocation);
+
   allocation->x = 0;
   allocation->y = 0;
+  allocation->width = gtk_widget_get_width (GTK_WIDGET (self));
+  allocation->height = gtk_widget_get_height (GTK_WIDGET (self));
 
   return TRUE;
 }
@@ -698,7 +700,7 @@ panel_dock_init (PanelDock *self)
   priv->controls = PANEL_MAXIMIZED_CONTROLS (panel_maximized_controls_new ());
   gtk_widget_set_halign (GTK_WIDGET (priv->controls), GTK_ALIGN_END);
   gtk_widget_set_valign (GTK_WIDGET (priv->controls), GTK_ALIGN_START);
-  gtk_widget_hide (GTK_WIDGET (priv->controls));
+  gtk_widget_set_visible (GTK_WIDGET (priv->controls), FALSE);
   gtk_overlay_add_overlay (priv->overlay, GTK_WIDGET (priv->controls));
 }
 
@@ -1483,7 +1485,7 @@ _panel_dock_set_maximized (PanelDock   *self,
     {
       gtk_widget_remove_css_class (GTK_WIDGET (priv->maximized), "maximized");
       gtk_overlay_remove_overlay (priv->overlay, GTK_WIDGET (priv->maximized));
-      gtk_widget_hide (GTK_WIDGET (priv->controls));
+      gtk_widget_set_visible (GTK_WIDGET (priv->controls), FALSE);
       priv->maximized = NULL;
     }
 
@@ -1500,7 +1502,7 @@ _panel_dock_set_maximized (PanelDock   *self,
       g_object_ref (priv->controls);
       gtk_overlay_remove_overlay (priv->overlay, GTK_WIDGET (priv->controls));
       gtk_overlay_add_overlay (priv->overlay, GTK_WIDGET (priv->controls));
-      gtk_widget_show (GTK_WIDGET (priv->controls));
+      gtk_widget_set_visible (GTK_WIDGET (priv->controls), TRUE);
       panel_widget_focus_default (widget);
       g_object_unref (priv->controls);
     }
@@ -1592,8 +1594,8 @@ _panel_dock_remove_frame (PanelDock  *self,
 /**
  * panel_dock_foreach_frame:
  * @self: a #PanelDock
- * @callback: (not nullable) (scope call): a function to be called on each frame
- * @user_data: (closure callback): data to pass to @callback
+ * @callback: (not nullable) (scope call) (closure user_data): a function to be called on each frame
+ * @user_data: data to pass to @callback
  *
  * Invokes a callback for each frame in the dock.
  */
